@@ -6,7 +6,7 @@
 #include "utils/numeric.h"
 #include "utils/formatting.h"
 #include <sys/time.h>
-#include "orafunc.h"
+#include "orafce.h"
 #include "builtins.h"
 
 #define ENABLE_INTERNATIONALIZED_WEEKDAY
@@ -590,15 +590,17 @@ _ora_date_round(DateADT day, int f)
 Datum ora_to_date(PG_FUNCTION_ARGS)
 {
 	text *date_txt = PG_GETARG_TEXT_P(0);
-	Timestamp newDate, result;
+	Timestamp result;
 
 	if(nls_date_format && strlen(nls_date_format))
 	{
+		Datum newDate;
+
 		/* it will return timestamp at GMT */
-		newDate = DatumGetTimestamp(DirectFunctionCall2(to_timestamp,
-											CStringGetDatum(date_txt),
-											CStringGetDatum(
-											cstring_to_text(nls_date_format))));
+		newDate = DirectFunctionCall2(to_timestamp,
+							CStringGetDatum(date_txt),
+							CStringGetDatum(cstring_to_text(nls_date_format)));
+
 		/* convert to local timestamp */
 		result = DatumGetTimestamp(DirectFunctionCall1(timestamptz_timestamp, newDate));
 	}
@@ -649,7 +651,12 @@ ora_timestamptz_trunc(PG_FUNCTION_ARGS)
 	int tz;
 	fsec_t fsec;
 	struct pg_tm tt, *tm = &tt;
+
+#if PG_VERSION_NUM >= 90200
 	const char *tzn;
+#else
+	char *tzn;
+#endif
 	bool redotz = false;
 	int f;
 
@@ -751,7 +758,13 @@ ora_timestamptz_round(PG_FUNCTION_ARGS)
 	int tz;
 	fsec_t fsec;
 	struct pg_tm tt, *tm = &tt;
+
+#if PG_VERSION_NUM >= 90200
 	const char *tzn;
+#else
+	char *tzn;
+#endif
+
 	bool redotz = false;
 	bool rounded = true;
 	int f;
