@@ -1,5 +1,8 @@
 /* contrib/orafce--3.0.sql */
 
+-- complain if script is sourced in psql, rather than via CREATE EXTENSION
+\echo Use "CREATE EXTENSION orafce" to load this file. \quit
+
 CREATE FUNCTION pg_catalog.trunc(value date, fmt text)
 RETURNS date
 AS 'MODULE_PATHNAME','ora_date_trunc'
@@ -156,6 +159,16 @@ AS 'MODULE_PATHNAME','orafce_to_number'
 LANGUAGE C IMMUTABLE STRICT;
 COMMENT ON FUNCTION pg_catalog.to_number(text) IS 'Convert string to number';
 
+CREATE OR REPLACE FUNCTION pg_catalog.to_number(numeric)
+RETURNS numeric AS $$
+SELECT pg_catalog.to_number($1::text);
+$$ LANGUAGE SQL IMMUTABLE;
+
+CREATE OR REPLACE FUNCTION pg_catalog.to_number(numeric,numeric)
+RETURNS numeric AS $$
+SELECT pg_catalog.to_number($1::text,$2::text);
+$$ LANGUAGE SQL IMMUTABLE;
+
 CREATE FUNCTION pg_catalog.to_date(str text)
 RETURNS timestamp
 AS 'MODULE_PATHNAME','ora_to_date'
@@ -255,48 +268,68 @@ AS 'MODULE_PATHNAME','oracle_substr3'
 LANGUAGE C IMMUTABLE STRICT;
 COMMENT ON FUNCTION oracle.substr(text, int, int) IS 'Returns substring started on start_in len chars';
 
+CREATE OR REPLACE FUNCTION oracle.substr(numeric,numeric)
+RETURNS text AS $$
+SELECT oracle.substr($1::text,trunc($2)::int);
+$$ LANGUAGE SQL IMMUTABLE;
+
+CREATE OR REPLACE FUNCTION oracle.substr(numeric,numeric,numeric)
+RETURNS text AS $$
+SELECT oracle.substr($1::text,trunc($2)::int,trunc($3)::int);
+$$ LANGUAGE SQL IMMUTABLE;
+
+CREATE OR REPLACE FUNCTION oracle.substr(varchar,numeric)
+RETURNS text AS $$
+SELECT oracle.substr($1,trunc($2)::int);
+$$ LANGUAGE SQL IMMUTABLE;
+
+CREATE OR REPLACE FUNCTION oracle.substr(varchar,numeric,numeric)
+RETURNS text AS $$
+SELECT oracle.substr($1,trunc($2)::int,trunc($3)::int);
+$$ LANGUAGE SQL IMMUTABLE;
+
 --can't overwrite PostgreSQL DATE data type!!!
 
 CREATE DOMAIN oracle.date AS timestamp(0);
 
 CREATE OR REPLACE FUNCTION oracle.add_days_to_timestamp(oracle.date,integer)
 RETURNS timestamp AS $$
-SELECT $1 + interval '1' day * $2;
+SELECT $1 + interval '1 day' * $2;
 $$ LANGUAGE SQL IMMUTABLE;
 
 CREATE OR REPLACE FUNCTION oracle.subtract (oracle.date, integer)
 RETURNS timestamp AS $$
-SELECT $1 - interval '1' day * $2;
+SELECT $1 - interval '1 day' * $2;
 $$ LANGUAGE SQL IMMUTABLE;
 
 CREATE OR REPLACE FUNCTION oracle.add_days_to_timestamp(oracle.date,bigint)
 RETURNS timestamp AS $$
-SELECT $1 + interval '1' day * $2;
+SELECT $1 + interval '1 day' * $2;
 $$ LANGUAGE SQL IMMUTABLE;
 
 CREATE OR REPLACE FUNCTION oracle.subtract (oracle.date, bigint)
 RETURNS timestamp AS $$
-SELECT $1 - interval '1' day * $2;
+SELECT $1 - interval '1 day' * $2;
 $$ LANGUAGE SQL IMMUTABLE;
 
 CREATE OR REPLACE FUNCTION oracle.add_days_to_timestamp(oracle.date,smallint)
 RETURNS timestamp AS $$
-SELECT $1 + interval '1' day * $2;
+SELECT $1 + interval '1 day' * $2;
 $$ LANGUAGE SQL IMMUTABLE;
 
 CREATE OR REPLACE FUNCTION oracle.subtract (oracle.date, smallint)
 RETURNS timestamp AS $$
-SELECT $1 - interval '1' day * $2;
+SELECT $1 - interval '1 day' * $2;
 $$ LANGUAGE SQL IMMUTABLE;
 
 CREATE OR REPLACE FUNCTION oracle.add_days_to_timestamp(oracle.date,numeric)
 RETURNS timestamp AS $$
-SELECT $1 + interval '1' day * $2;
+SELECT $1 + interval '1 day' * $2;
 $$ LANGUAGE SQL IMMUTABLE;
 
 CREATE OR REPLACE FUNCTION oracle.subtract (oracle.date, numeric)
 RETURNS timestamp AS $$
-SELECT $1 - interval '1' day * $2;
+SELECT $1 - interval '1 day' * $2;
 $$ LANGUAGE SQL IMMUTABLE;
 
 CREATE OR REPLACE FUNCTION oracle.subtract(oracle.date,oracle.date)
@@ -360,7 +393,7 @@ CREATE OPERATOR oracle.- (
 
 CREATE FUNCTION oracle.add_months(TIMESTAMP WITH TIME ZONE,INTEGER)
 RETURNS TIMESTAMP
-AS $$ SELECT ($1 + interval '1' month * $2)::oracle.date; $$
+AS $$ SELECT ($1 + interval '1 month' * $2)::oracle.date; $$
 LANGUAGE SQL IMMUTABLE STRICT;
 
 CREATE OR REPLACE FUNCTION oracle.last_day(TIMESTAMPTZ)
@@ -1994,6 +2027,383 @@ CREATE AGGREGATE pg_catalog.median(double precision) (
   FINALFUNC=pg_catalog.median8_finalfn
 );
 
+-- oracle.varchar2 type support
+
+CREATE FUNCTION varchar2in(cstring,oid,integer)
+RETURNS varchar2
+AS 'MODULE_PATHNAME','varchar2in'
+LANGUAGE C
+STRICT
+IMMUTABLE;
+
+CREATE FUNCTION varchar2out(varchar2)
+RETURNS CSTRING
+AS 'MODULE_PATHNAME','varchar2out'
+LANGUAGE C
+STRICT
+IMMUTABLE;
+
+CREATE FUNCTION varchar2_transform(internal)
+RETURNS internal
+AS 'varchar_transform'
+LANGUAGE internal
+STRICT
+IMMUTABLE;
+
+CREATE FUNCTION varchar2recv(internal,oid,integer)
+RETURNS varchar2
+AS 'MODULE_PATHNAME','varchar2recv'
+LANGUAGE C
+STRICT
+STABLE;
+
+CREATE FUNCTION varchar2send(varchar2)
+RETURNS bytea
+AS 'varcharsend'
+LANGUAGE internal
+STRICT
+STABLE;
+
+CREATE FUNCTION varchar2typmodin(cstring[])
+RETURNS integer
+AS 'varchartypmodin'
+LANGUAGE internal
+STRICT
+IMMUTABLE;
+
+CREATE FUNCTION varchar2typmodout(integer)
+RETURNS CSTRING
+AS 'varchartypmodout'
+LANGUAGE internal
+STRICT
+IMMUTABLE;
+
+CREATE FUNCTION varchar2(varchar2,integer,boolean)
+RETURNS varchar2
+AS 'MODULE_PATHNAME','varchar2'
+LANGUAGE C
+STRICT
+IMMUTABLE;
+
+/* CREATE TYPE */
+CREATE TYPE varchar2 (
+internallength = VARIABLE,
+input = varchar2in,
+output = varchar2out,
+receive = varchar2recv,
+send = varchar2send,
+category = 'S',
+typmod_in = varchar2typmodin,
+typmod_out = varchar2typmodout
+);
+
+/* CREATE CAST */
+CREATE CAST (varchar2 AS text)
+WITHOUT FUNCTION
+AS IMPLICIT;
+
+CREATE CAST (text AS varchar2)
+WITHOUT FUNCTION
+AS IMPLICIT;
+
+CREATE CAST (varchar2 AS char)
+WITHOUT FUNCTION
+AS IMPLICIT;
+
+CREATE CAST (char AS varchar2)
+WITHOUT FUNCTION
+AS IMPLICIT;
+
+CREATE CAST (varchar2 AS varchar)
+WITHOUT FUNCTION
+AS IMPLICIT;
+
+CREATE CAST (varchar AS varchar2)
+WITHOUT FUNCTION
+AS IMPLICIT;
+
+CREATE CAST (varchar2 AS varchar2)
+WITH FUNCTION varchar2(varchar2,integer,boolean)
+AS IMPLICIT;
+
+CREATE CAST (varchar2 AS real)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (real AS varchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (varchar2 AS double precision)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (double precision AS varchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (varchar2 AS integer)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (integer AS varchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (varchar2 AS smallint)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (smallint AS varchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (varchar2 AS bigint)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (bigint AS varchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (varchar2 AS numeric)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (numeric AS varchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (varchar2 AS date)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (date AS varchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (varchar2 AS timestamp)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (timestamp AS varchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (varchar2 AS interval)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (interval AS varchar2)
+WITH INOUT
+AS IMPLICIT;
+
+UPDATE pg_proc
+SET protransform=(SELECT oid FROM pg_proc WHERE proname='varchar2_transform')
+WHERE proname='varchar2';
+
+-- string functions for varchar2 type
+-- these are 'byte' versions of corresponsing text/varchar functions
+
+CREATE OR REPLACE FUNCTION pg_catalog.substrb(varchar2, integer, integer) RETURNS varchar2
+AS 'bytea_substr'
+LANGUAGE internal
+STRICT IMMUTABLE;
+COMMENT ON FUNCTION pg_catalog.substrb(varchar2, integer, integer) IS 'extracts specified number of bytes from the input varchar2 string starting at the specified byte position (1-based) and returns as a varchar2 string';
+
+CREATE OR REPLACE FUNCTION pg_catalog.substrb(varchar2, integer) RETURNS varchar2
+AS 'bytea_substr_no_len'
+LANGUAGE internal
+STRICT IMMUTABLE;
+COMMENT ON FUNCTION pg_catalog.substrb(varchar2, integer, integer) IS 'extracts specified number of bytes from the input varchar2 string starting at the specified byte position (1-based) and returns as a varchar2 string';
+
+CREATE OR REPLACE FUNCTION pg_catalog.lengthb(varchar2) RETURNS integer
+AS 'byteaoctetlen'
+LANGUAGE internal
+STRICT IMMUTABLE;
+COMMENT ON FUNCTION pg_catalog.lengthb(varchar2) IS 'returns byte length of the input varchar2 string';
+
+CREATE OR REPLACE FUNCTION pg_catalog.strposb(varchar2, varchar2) RETURNS integer
+AS 'byteapos'
+LANGUAGE internal
+STRICT IMMUTABLE;
+COMMENT ON FUNCTION pg_catalog.strposb(varchar2, varchar2) IS 'returns the byte position of a specified string in the input varchar2 string';
+
+-- oracle.nvarchar2 type support
+
+CREATE FUNCTION nvarchar2in(cstring,oid,integer)
+RETURNS nvarchar2
+AS 'MODULE_PATHNAME','nvarchar2in'
+LANGUAGE C
+STRICT
+IMMUTABLE;
+
+CREATE FUNCTION nvarchar2out(nvarchar2)
+RETURNS CSTRING
+AS 'MODULE_PATHNAME','nvarchar2out'
+LANGUAGE C
+STRICT
+IMMUTABLE;
+
+CREATE FUNCTION nvarchar2_transform(internal)
+RETURNS internal
+AS 'varchar_transform'
+LANGUAGE internal
+STRICT
+IMMUTABLE;
+
+CREATE FUNCTION nvarchar2recv(internal,oid,integer)
+RETURNS nvarchar2
+AS 'MODULE_PATHNAME','nvarchar2recv'
+LANGUAGE C
+STRICT
+STABLE;
+
+CREATE FUNCTION nvarchar2send(nvarchar2)
+RETURNS bytea
+AS 'varcharsend'
+LANGUAGE internal
+STRICT
+STABLE;
+
+CREATE FUNCTION nvarchar2typmodin(cstring[])
+RETURNS integer
+AS 'varchartypmodin'
+LANGUAGE internal
+STRICT
+IMMUTABLE;
+
+CREATE FUNCTION nvarchar2typmodout(integer)
+RETURNS CSTRING
+AS 'varchartypmodout'
+LANGUAGE internal
+STRICT
+IMMUTABLE;
+
+CREATE FUNCTION nvarchar2(nvarchar2,integer,boolean)
+RETURNS nvarchar2
+AS 'MODULE_PATHNAME','nvarchar2'
+LANGUAGE C
+STRICT
+IMMUTABLE;
+
+/* CREATE TYPE */
+CREATE TYPE nvarchar2 (
+internallength = VARIABLE,
+input = nvarchar2in,
+output = nvarchar2out,
+receive = nvarchar2recv,
+send = nvarchar2send,
+category = 'S',
+typmod_in = nvarchar2typmodin,
+typmod_out = nvarchar2typmodout
+);
+
+/* CREATE CAST */
+CREATE CAST (nvarchar2 AS text)
+WITHOUT FUNCTION
+AS IMPLICIT;
+
+CREATE CAST (text AS nvarchar2)
+WITHOUT FUNCTION
+AS IMPLICIT;
+
+CREATE CAST (nvarchar2 AS char)
+WITHOUT FUNCTION
+AS IMPLICIT;
+
+CREATE CAST (char AS nvarchar2)
+WITHOUT FUNCTION
+AS IMPLICIT;
+
+CREATE CAST (nvarchar2 AS varchar)
+WITHOUT FUNCTION
+AS IMPLICIT;
+
+CREATE CAST (varchar AS nvarchar2)
+WITHOUT FUNCTION
+AS IMPLICIT;
+
+CREATE CAST (nvarchar2 AS nvarchar2)
+WITH FUNCTION nvarchar2(nvarchar2, integer, boolean)
+AS IMPLICIT;
+
+CREATE CAST (nvarchar2 AS real)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (real AS nvarchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (nvarchar2 AS double precision)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (double precision AS nvarchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (nvarchar2 AS integer)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (integer AS nvarchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (nvarchar2 AS smallint)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (smallint AS nvarchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (nvarchar2 AS bigint)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (bigint AS nvarchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (nvarchar2 AS numeric)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (numeric AS nvarchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (nvarchar2 AS date)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (date AS nvarchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (nvarchar2 AS timestamp)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (timestamp AS nvarchar2)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (nvarchar2 AS interval)
+WITH INOUT
+AS IMPLICIT;
+
+CREATE CAST (interval AS nvarchar2)
+WITH INOUT
+AS IMPLICIT;
+
+UPDATE pg_proc
+SET protransform=(SELECT oid FROM pg_proc WHERE proname='varchar2_transform')
+WHERE proname='nvarchar2';
+
 GRANT USAGE ON SCHEMA dbms_pipe TO PUBLIC;
 GRANT USAGE ON SCHEMA dbms_alert TO PUBLIC;
 GRANT USAGE ON SCHEMA plvdate TO PUBLIC;
